from microprediction.conventions import MicroConventions
import requests

class MicroClient(MicroConventions):

    def __init__(self,write_key="invalid_key", base_url="http://www.microprediction.com/"):
        """ Establish connection and adopt configuration parameters from site """
        if not self.is_vanity_key(write_key):
            print("No write_key was provided or it was invalid. Creating a new one but this may take "+ MicroConventions.new_vanity_key_estimate())
            self.write_key = MicroConventions.new_vanity_key()
            print("Your write_key is "+str(self.write_key)+' and we suggest you keep it safe.')
            write_code = MicroConventions.hash(self.write_key)
            spirit_word = write_code[:7].replace('5','s').replace('1','l').replace('0','o')
            print("Your mnemonic is "+spirit_word+", which is similar to the first seven characters of the hash of your write_key.")
        self.base_url  = base_url
        self.write_key = write_key
        config = requests.get(self.base_url+"/config.json").json()
        self.delays = config["delays"]
        self.num_predictions = config["num_predictions"]

    def set(self, name, write_key, value):
        """ Create or update a stream """
        res = requests.put(self.base_url + '/live/' + name, data={"write_key":write_key,"value":value})
        if res.status_code == 200:
            return float(res)

    def get(self, name):
        res = requests.get(self.base_url + '/live/' + name)
        if res.status_code == 200:
           return res.json()

    def get_current_value(self, name):
        res = requests.get(self.base_url + '/live/' + name)
        if res.status_code == 200:
           return float(res.json())

    def get_lagged_values(self, name):
        """ Retrieve lagged values of a time series
        :param name:    cop.json   z1~cop.json   z2~cop~qp.json
        :return: [ float ]
        """
        res = requests.get(self.base_url+'live/lagged_values::'+name )
        if res.status_code==200:
            return res.json()

    def get_lagged_times(self, name):
        """ Retrieve lagged times
        :param name:    cop.json   z1~cop.json   z2~cop~qp.json
        :return: [ float ]
        """
        res = requests.get(self.base_url+'live/lagged_times::'+name )
        if res.status_code==200:
            return res.json()

    def get_delayed_value(self, name, delay=None):
        """ Retrieve quarantined value
        :param name:    cop.json   z1~cop.json   z2~cop~qp.json
        :return: [ float ]
        """
        delay = delay or self.delays[0]
        res = requests.get(self.base_url + 'live/delayed::'+str(delay)+ "::" + name)
        if res.status_code == 200:
            return res.json()

    def get_errors(self):
        res = requests.get(self.base_url + 'live/errors::' + self.write_key + '.json')
        if res.status_code == 200:
            return res.json()

    def get_cdf(self, name, values=None):
        if values is None:  # Supply x-values at which approximate crowd cdf will be computed.
            values = [-2.3263478740408408, -1.6368267885518997, -1.330561513178897, -1.1146510149326596,
                      -0.941074530352976, -0.792046894425591, -0.6588376927361878, -0.5364223812298266,
                      -0.4215776353171568, -0.3120533220328322, -0.20615905948527324, -0.10253336200497987, 0.0,
                      0.10253336200497973, 0.20615905948527324, 0.31205332203283237, 0.4215776353171568,
                      0.5364223812298264, 0.6588376927361878, 0.7920468944255913, 0.941074530352976, 1.1146510149326592,
                      1.330561513178897, 1.6368267885519001, 2.3263478740408408]
        comma_sep_values = ",".join([str(v) for v in values])
        res = requests.get(self.base_url + 'live/' + name, params={"values", comma_sep_values})
        if res.status_code == 200:
            return res.json()

    def get_balance(self):
        res = requests.get(self.base_url + 'live/errors::' + self.write_key)
        if res.status_code == 200:
            return res.json()

    def submit(self, name, values):
        """ Submit prediction scenarios
        :param name:      str         Examples:    cop.json   z1~cop.json   z2~cop~qp.json
        :param write_key: str         Example:    "5263ee89-e34e-44dc-8b91-445b302b043e"
        :param values:    [ float ]
        :return: bool
        """
        assert len(values)==self.num_predictions
        comma_sep_values = ",".join([ str(v) for v in values ] )
        res = requests.put(self.base_url + 'submit/' + name, data={'delay':self.delays[0], 'write_key': self.write_key, 'values': comma_sep_values})
        return res.status_code==200

    def withdraw(self, name ):
        """ Withdraw predictions """
        codes = list()
        for delay in self.delays:
            res = requests.delete(self.base_url + 'submit/'+name, params={'delay':delay} )
            codes.append(res.status_code)
        return all([ c==200 for c in codes ] )




import uuid

class MicroConventions(object):

    @staticmethod
    def strength():
        return 9

    @staticmethod
    def random_key():
        return str(uuid.uuid4())

    @staticmethod
    def random_name():
        return MicroConventions.random_key() + '.json'

    @staticmethod
    def hash(key):
        return str(uuid.uuid5(uuid.NAMESPACE_DNS, key))

    @staticmethod
    def dictionary9():
        return [ 'baldfaced','boldfaced','coalfaces','decodable',
                 'accolades','caboodles','closeable','coalesced',
                 'ecolabels','scaleable','calaboose','deadballs',
                 'deadfalls','escaladed','looseleaf']

    @staticmethod
    def dictionary7():
        return [ 'albedoe','beefalo','befools','debacle','beadles','beefalo',
                 'caboodl','befleas','belaced','boodles','caboose','codable',
                 'doolees','elodeas','solaced','sofabed','seafood','fadable',
                 'abodes0','cabals0','f1eeced','feedab1','1abe1ed','facaded',
                 'defaced','daffed0','cabbed0','baff1ed','acceded','2faced0',
                 'faded00','beaded0','decaded','ebbed00','acceded','effaced',
                 'ceded00','dada007','debac1e','decada1','1eafed0','fab1ed00',
                 'dabb1e0','cab1ed0','1ead000','b1ade00','f1ea000','flee000',
                 'beef000','babe000','ee10000','dad0000','deadfa11','fadab1e',
                 '1ad1ed0','be1aced','class00','leadles','fleeced','decease',
                 'sealed0','scales0','felled0','leafles','eased00','decaf00',
                 'called0','sell000','self000']

    @staticmethod
    def is_english_word(word,dictionary=None):
        dictionary = dictionary or MicroConventions.default_dictionary()
        return word in dictionary

    @staticmethod
    def default_dictionary():
        return MicroConventions.dictionary7() if MicroConventions.strength()==7 else MicroConventions.dictionary9()

    @staticmethod
    def is_vanity_key(write_key=None,write_code=None,dictionary=None):
        dictionary = dictionary or MicroConventions.default_dictionary()
        write_code = write_code or MicroConventions.hash(write_key)
        candidate_word = MicroConventions.to_mnemonic(write_code=write_code)
        return MicroConventions.is_english_word(candidate_word, dictionary=dictionary)

    @staticmethod
    def new_vanity_key_estimate():
        # Can take quite a long time :)
        return "up to ten minutes or so " if MicroConventions.strength()==7 else " literally all day."

    @staticmethod
    def new_vanity_key(dictionary=None):
        # Can take quite a long time :)
        remaining = int(1e9)
        dictionary = dictionary or MicroConventions.default_dictionary()
        while remaining:
            remaining = remaining - 1
            write_key = MicroConventions.random_key()
            if MicroConventions.is_vanity_key(write_key=write_key, dictionary=dictionary):
                return write_key
        raise Exception('Failed to generate vanity key')

    @staticmethod
    def to_mnemonic(write_key=None, write_code=None):
        write_code = write_code or MicroConventions.hash(write_key)
        return write_code[:7].replace('1','l').replace('0','o').replace('5','s')

